unit adxOLFolderContextMenu_IMPL;

interface

uses
  SysUtils, ComObj, ComServ, ActiveX, Variants, Outlook2000, Office2000,
  adxAddIn, adxOLFolderContextMenu_TLB, StdVcl;

type
  TADX_OLFolderContextMenu = class(TadxAddin, IADX_OLFolderContextMenu)
  end;

  TAddInModule = class(TadxCOMAddInModule)
    procedure adxCOMAddInModuleAddInInitialize(Sender: TObject);
    procedure adxCOMAddInModuleAddInFinalize(Sender: TObject);
  private
    FSink: Pointer; // TEventSink
    FContextBar: TadxCommandBar;
  protected
  public
    procedure DoCommandBarsUpdate;
    procedure DoClick(Sender: TObject);
  end;

implementation

{$R *.dfm}

uses Windows, Dialogs;

type
  ICmdBarsEvents = interface(IDispatch)
    ['{000C0352-0000-0000-C000-000000000046}']
  end;

  TEventSink = class(TInterfacedObject, ICmdBarsEvents)
  private
    FCookie: Longint;
    FCmdBars: CommandBars;
    FModule: TAddInModule;
    { IDispatch }
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;
  public
    constructor Create(AddInModule: TAddInModule);
    destructor Destroy; override;
    procedure Disconnect;
  end;                                     

{ TEventSink }

constructor TEventSink.Create(AddInModule: TAddInModule);
begin
  inherited Create;
  FModule := AddInModule;
  FCmdBars := FModule.OutlookApp.ActiveExplorer.CommandBars;
  InterfaceConnect(FCmdBars, DIID__CommandBarsEvents, Self, FCookie);
end;

destructor TEventSink.Destroy;
begin
  FCmdBars := nil;
  FModule := nil;
  inherited Destroy;
end;

procedure TEventSink.Disconnect;
begin
  try
    InterfaceDisconnect(FCmdBars, DIID__CommandBarsEvents, FCookie);
  except
  end;
end;

function TEventSink.GetTypeInfoCount(out Count: Integer): HResult;
begin
  Count := 0;
  Result := S_OK;
end;

function TEventSink.GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult;
begin
  Pointer(TypeInfo) := nil;
  Result := E_FAIL;
end;

function TEventSink.GetIDsOfNames(const IID: TGUID; Names: Pointer;
  NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
var
  s: string;
  i: integer;
begin
  Result := S_OK;
  if DispIDs = nil then Result := E_POINTER;
  if (Result = S_OK) and (Names = nil) then Result := E_POINTER;
  if (Result = S_OK) and (NameCount <> 1) then Result := E_INVALIDARG;
  if Result = S_OK then begin
    i := 512;
    SetLength(s, i);
    try
      WideCharToMultiByte(CP_ACP, 0, Names, -1, PChar(s), i, nil, nil);
      i := lstrcmp(PChar(s), 'OnUpdate');
      if i <> 0 then Result := DISP_E_UNKNOWNNAME
      else PInteger(DispIDs)^ := $00000001;
    finally
      SetLength(s, 0);
    end;
  end;
end;

function TEventSink.Invoke(DispID: Integer; const IID: TGUID;
  LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo,
  ArgErr: Pointer): HResult;
begin
  Result := S_OK;
  if DispID <> 1 then Result := DISP_E_MEMBERNOTFOUND;
  if Result = S_OK then
    try
      FModule.DoCommandBarsUpdate;
    except
    end;
end;

{ TAddInModule }

procedure TAddInModule.adxCOMAddInModuleAddInInitialize(Sender: TObject);
begin
  FContextBar := nil;
  FSink := TEventSink.Create(Self);
end;

procedure TAddInModule.adxCOMAddInModuleAddInFinalize(Sender: TObject);
begin
  TEventSink(FSink).Disconnect;
  FSink := nil;
  if Assigned(FContextBar) then FreeAndNil(FContextBar);
end;

procedure TAddInModule.DoCommandBarsUpdate;
var
  ContextBar: CommandBar;
  NewButton: CommandBarControl;
  OldProtection: TOleEnum;
begin
  ContextBar := FindCommandBar('Folder Context Menu');
  if Assigned(ContextBar) and (ContextBar.Controls.Count > 0) then
    try
      NewButton := ContextBar.FindControl(msoControlButton, EmptyParam, 'MY_UNIQUE_TAG', EmptyParam, EmptyParam) as CommandBarButton;
      if NewButton = nil then begin
        OldProtection := ContextBar.Protection;
        ContextBar.Set_Protection(msoBarNoProtection);

        if FContextBar = nil then begin
          FContextBar := TadxCommandBar.Create(Self);
          FContextBar.CommandBarName := 'Folder Context Menu';
        end
        else
          FContextBar.Controls.Clear;
          
        with FContextBar.Controls.Add(adxButton, 'MY_UNIQUE_TAG', 0, True).AsButton do
        begin
          BeginGroup := True;
          Style := adxMsoButtonCaption;
          Caption := 'MyButton';
          OnClick := DoClick;
        end;
        ContextBar.Set_Protection(OldProtection);
      end;
    finally
      NewButton := nil;
      ContextBar := nil;
    end;
end;

procedure TAddInModule.DoClick(Sender: TObject);
begin
  ShowMessage(TadxCommandBarControl(Sender).AsButton.Caption);
end;

initialization
  TadxFactory.Create(ComServer, TADX_OLFolderContextMenu, CLASS_ADX_OLFolderContextMenu, TAddInModule);

end.
